
/******************************************************************************
* MODULE     : mmlout.hpp
* DESCRIPTION: Standard output facilities and switches
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_MMLOUT_HPP
#define __MML_MMLOUT_HPP
namespace mml {

class string;
class integer;
class rational;
class dyadic;
class interval;

/******************************************************************************
* Printing expressions
******************************************************************************/

extern void* as_generic_aux_ptr;

template<typename C> string print_to_string (const C& x);
template<typename C> string print_to_lisp (const C& x);

template<typename C> generic
as_generic (const C& x) {
  if (as_generic_aux_ptr == (void*) &x)
    return print_to_string (x);
  else {
    void* old= as_generic_aux_ptr;
    as_generic_aux_ptr= (void*) &x;
    generic r= from_lisp (print_to_lisp (x));
    as_generic_aux_ptr= old;
    return r;
  }
}

generic as_generic (int i);
generic as_generic (nat i);
generic as_generic (unsigned long int i);
generic as_generic (signed long int i);
generic as_generic (float x);
generic as_generic (double x);
generic as_generic (char* s);
generic as_generic (const string& s);
generic as_generic (const integer& i);
generic as_generic (const rational& r);
generic as_generic (const generic& r);

template<typename T> inline generic
as_generic (char* op, const T& t) {
  return generic (op) (as_generic (t));
}

template<typename T, typename U> inline generic
as_generic (char* op, const T& t, const U& u) {
  return generic (op) (as_generic (t), as_generic (u));
}

template<typename C> inline ostream&
print_lisp (ostream& out, const C& x) {
  lisp_print (out, as_generic (x));
  return out;
}

#define PRINT_SUGAR(TMPL,T)				\
TMPL inline ostream& operator << (ostream& out, const T& x) {	\
  return out << as_generic (x); }

/******************************************************************************
* Output switches
******************************************************************************/

template<typename T>
class output_switch {
public:
  T& var;
  T  val;
  output_switch (T& var2, T val2): var (var2), val (val2) {}
};

template<typename T> inline ostream&
operator << (ostream& o, const output_switch<T>& x) {
  x.var= x.val; return o;
}

extern bool     interval_by_bounds;    // display intervals as pairs of bounds
extern nat      the_series_prec;       // series precision for printing
extern nat      the_series_zt;         // series precision for zero-testing
extern bool     the_series_expr;       // display series as expressions

generic& the_series_var ();     // name for the series variable
generic& the_polynomial_var (); // name for the polynomial variable

inline output_switch<bool> set_interval_by_bounds (bool flag= false) {
  return output_switch<bool> (interval_by_bounds, flag); }
inline output_switch<nat> set_series_prec (nat n= 10) {
  return output_switch<nat> (the_series_prec, n); }
inline output_switch<nat> set_series_zt (nat n= 25) {
  return output_switch<nat> (the_series_zt, n); }
inline output_switch<generic> set_series_var (const generic& name= "z") {
  return output_switch<generic> (the_series_var (), name); }
inline output_switch<bool> set_series_expr (bool flag= false) {
  return output_switch<bool> (the_series_expr, flag); }
inline output_switch<generic> set_polynomial_var (const generic& name= "x") {
  return output_switch<generic> (the_polynomial_var (), name); }

} // namespace mml
#endif // __MML_MMLOUT_HPP
