
/******************************************************************************
* MODULE     : interval_acos.cpp
* DESCRIPTION: Compute the arc cosine of an interval
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
acos (const interval& x) {
  if (is_error (x)) return x;
  if (has_low_prec (x)) {
    interval_rep* r= raw_interval ();
    mpfr_acos (r->min, x->max, GMP_RNDD);
    mpfr_acos (r->max, x->min, GMP_RNDU);
    return r;
  }
  else {
    interval_rep* r= raw_interval (prec (x));
    direct_dyadic aux, err;
    mpfr_acos (r->mid, x->mid, GMP_RNDN);
    CHECK_ERROR (r);
    mpfr_mul (*aux, x->mid, x->mid, GMP_RNDU);
    mpfr_set_si (*err, 1, GMP_RNDD);
    mpfr_sub (*aux, *err, *aux, GMP_RNDD);
    if (mpfr_sgn (*aux) == 0) {
      direct_dyadic tmp (prec (x)), one (prec (x));
      mpfr_set_si (*one, 1, GMP_RNDD);
      mpfr_abs (*tmp, x->mid, GMP_RNDU);
      mpfr_sub (*aux, *one, *tmp, GMP_RNDD);
      mpfr_set_si (*err, 2, GMP_RNDD);
      mpfr_sub (*err, *err, *aux, GMP_RNDD);
      mpfr_mul (*aux, *aux, *err, GMP_RNDD);
    }
    mpfr_sqrt (*aux, *aux, GMP_RNDD);
    mpfr_div (r->rad, x->rad, *aux, GMP_RNDU);
    mpfr_set_ui_2si (*err, 2, mpfr_get_step (r->mid), GMP_RNDU);
    mpfr_add (r->rad, r->rad, *err, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
