
/******************************************************************************
* MODULE     : interval_atan.cpp
* DESCRIPTION: Compute the arc tangent of an interval
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
atan (const interval& x) {
  if (is_error (x)) return x;
  if (has_low_prec (x)) {
    interval_rep* r= raw_interval ();
    mpfr_atan (r->min, x->min, GMP_RNDD);
    mpfr_atan (r->max, x->max, GMP_RNDU);
    return r;
  }
  else {
    mp_exp_t gain= max (expo (x), (mp_exp_t) 0) + ((mp_exp_t) 1);
    if (gain > 100) gain= 100; // avoid unneccessarily precise computations
    interval_rep* r= raw_interval (prec (x) + gain);
    direct_dyadic aux, err;
    mpfr_atan (r->mid, x->mid, GMP_RNDN);
    mpfr_mul (*aux, x->mid, x->mid, GMP_RNDD);
    mpfr_set_si (*err, 1, GMP_RNDD);
    mpfr_add (*aux, *aux, *err, GMP_RNDD);
    mpfr_div (r->rad, x->rad, *aux, GMP_RNDU);
    mpfr_set_ui_2si (*err, 2, mpfr_get_step (r->mid), GMP_RNDU);
    mpfr_add (r->rad, r->rad, *err, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
