
/******************************************************************************
* MODULE     : interval_atan2.cpp
* DESCRIPTION: Variant of atan for computing the argument of a complex number.
*              Notice that upper limit of the result may be larger than pi
*              if the complex interval intersects the negative real axis.
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
atan2 (const interval& y, const interval& x) {
  mp_prec_t p= max (prec (x), prec (y));
  if (sign (lower (x)) > 0) return atan (y/x);
  if (sign (lower (y)) > 0) return atan (-x/y) + (interval::pi (p) >> 1);
  if (sign (upper (y)) < 0) return atan (-x/y) - (interval::pi (p) >> 1);
  if (sign (upper (x)) < 0) return atan (y/x) + interval::pi (p);
  dyadic pi= next_above (dyadic::pi (DEFAULT_LOW));
  return segment (-pi, pi);
}

} // namespace mml
