
/******************************************************************************
* MODULE     : interval_bound.cpp
* DESCRIPTION: Bound for the absolute value the interval
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

dyadic
bound (const interval& x) {
  dyadic r (DEFAULT_LOW);
  if (has_low_prec (x)) {
    direct_dyadic aux1, aux2;
    mpfr_abs (*aux1, x->min, GMP_RNDU);
    mpfr_abs (*aux2, x->min, GMP_RNDU);
    if (mpfr_cmp (*aux1, *aux2) >= 0)
      mpfr_set (*r, *aux1, GMP_RNDU);
    else mpfr_set (*r, *aux2, GMP_RNDU);
  }
  else {
    mpfr_abs (*r, x->mid, GMP_RNDU);
    mpfr_add (*r, *r, x->rad, GMP_RNDU);
  }
  return r;
}

} // namespace mml
