
/******************************************************************************
* MODULE     : interval_constants.cpp
* DESCRIPTION: Various mathematical constants
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
constant_interval (mpfr_cst_algo mpfr_algo, mp_prec_t prec) {
  if (prec <= DEFAULT_LOW) {
    interval_rep* r= raw_interval (prec);
    mpfr_algo (r->min, GMP_RNDD);
    mpfr_algo (r->max, GMP_RNDU);
    return r;
  }
  else {
    interval_rep* r= raw_interval (prec);
    mpfr_algo (r->mid, GMP_RNDN);
    mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
    return r->finish_high ();
  }
}

interval interval::log2 (mp_prec_t prec) {
  return constant_interval (mpfr_const_log2, prec); }
interval interval::pi (mp_prec_t prec) {
  return constant_interval (mpfr_const_pi, prec); }
interval interval::euler (mp_prec_t prec) {
  return constant_interval (mpfr_const_euler, prec); }
//interval interval::catalan (mp_prec_t prec) {
//  return constant_interval (mpfr_const_catalan, prec); }

} // namespace mml
