
/******************************************************************************
* MODULE     : interval_sin.cpp
* DESCRIPTION: Compute the cosine of an interval
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
cos (const interval& x) {
  if (is_error (x)) return x;
  if (is_infinity (x))
    return segment (sint_lo (-1, DEFAULT_LOW), sint_up (1, DEFAULT_LOW));
  else if (has_low_prec (x)) {
    interval_rep* r= raw_interval ();
    direct_dyadic d, pi;
    mpfr_sub (*d, x->max, x->min, GMP_RNDU);
    mpfr_const_pi (*pi, GMP_RNDD);
    if (mpfr_cmp (*d, *pi) >= 0) {
      mpfr_set_si (r->min, -1, GMP_RNDD);
      mpfr_set_si (r->max, 1, GMP_RNDU);
    }
    else {
      direct_dyadic sin_min, sin_max;
      mpfr_sin (*sin_min, x->min, GMP_RNDN);
      mpfr_sin (*sin_max, x->max, GMP_RNDN);
      if (mpfr_sgn (*sin_min) <= 0) {
	if (mpfr_sgn (*sin_max) <= 0) {
	  mpfr_cos (r->min, x->min, GMP_RNDD);
	  mpfr_cos (r->max, x->max, GMP_RNDU);
	}
	else {
	  mpfr_cos (r->min, x->min, GMP_RNDD);
	  mpfr_cos (r->max, x->max, GMP_RNDD);
	  mpfr_min (r->min, r->min, r->max, GMP_RNDD);
	  mpfr_set_si (r->max, 1, GMP_RNDU);
	}
      }
      else {
	if (mpfr_sgn (*sin_max) <= 0) {
	  mpfr_cos (r->min, x->min, GMP_RNDU);
	  mpfr_cos (r->max, x->max, GMP_RNDU);
	  mpfr_min (r->max, r->min, r->max, GMP_RNDU);
	  mpfr_set_si (r->min, -1, GMP_RNDD);
	}
	else {
	  mpfr_cos (r->min, x->max, GMP_RNDD);
	  mpfr_cos (r->max, x->min, GMP_RNDU);
	}
      }
    }
    return r;
  }
  else {
    mp_exp_t loss= max ((mp_exp_t) expo (x) - 3, (mp_exp_t) 0);
    interval_rep* r= raw_interval (prec (x) - loss);
    direct_dyadic err;
    mpfr_cos (r->mid, x->mid, GMP_RNDN);
    mpfr_sin (*err, x->mid, GMP_RNDN);
    mpfr_abs (*err, *err, GMP_RNDU);
    mpfr_mul (*err, *err, x->rad, GMP_RNDU);
    mpfr_set_ui_2si (r->rad, 2, mpfr_get_step (x->mid), GMP_RNDU);
    mpfr_add (r->rad, r->rad, *err, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
