
/******************************************************************************
* MODULE     : interval_div.cpp
* DESCRIPTION: Division of intervals
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
operator / (const interval& x, const interval& y) {
  if (is_error (x)) return x;
  if (is_error (y)) return y;

  if (has_low_prec (x) || has_low_prec (y)) {
    interval X= reduce (x);
    interval Y= reduce (y);
    interval_rep* r= raw_interval ();
    if (mpfr_sgn (Y->min) > 0) {
      if (mpfr_sgn (X->min) > 0) mpfr_div (r->min, X->min, Y->max, GMP_RNDD);
      else mpfr_div (r->min, X->min, Y->min, GMP_RNDD);
      if (mpfr_sgn (X->max) > 0) mpfr_div (r->max, X->max, Y->min, GMP_RNDU);
      else mpfr_div (r->max, X->max, Y->max, GMP_RNDU);
    }
    else if (mpfr_sgn (Y->max) < 0) {
      if (mpfr_sgn (X->max) > 0) mpfr_div (r->min, X->max, Y->max, GMP_RNDD);
      else mpfr_div (r->min, X->max, Y->min, GMP_RNDD);
      if (mpfr_sgn (X->min) > 0) mpfr_div (r->max, X->min, Y->min, GMP_RNDU);
      else mpfr_div (r->max, X->min, Y->max, GMP_RNDU);
    }
    else return r->finish_error ();
    return r;
  }

  else {
    interval_rep* r= raw_interval (min (prec (x), prec (y)) + 1);
    direct_dyadic aux, abs_x, abs_y, rel_x, rel_y, ser, one, temp;
    mpfr_set_si (*one, 1, GMP_RNDN);
    mpfr_div (r->mid, x->mid, y->mid, GMP_RNDN);
    CHECK_UNDERFLOW (r);
    CHECK_OVERFLOW (r);
    mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
    mpfr_abs (*abs_x, x->mid, GMP_RNDU);
    mpfr_div (*rel_x, x->rad, *abs_x, GMP_RNDU);
    mpfr_abs (*abs_y, y->mid, GMP_RNDU);
    mpfr_div (*rel_y, y->rad, *abs_y, GMP_RNDU);
    mpfr_sub (*aux, *one, *rel_y, GMP_RNDD);
    mpfr_div (*ser, *one, *aux, GMP_RNDU);
    mpfr_add (*aux, *rel_x, *rel_y, GMP_RNDU);
    mpfr_mul (*aux, *aux, *ser, GMP_RNDU);
    mpfr_mul (*aux, *aux, *abs_x, GMP_RNDU);
    mpfr_div (*aux, *aux, *abs_y, GMP_RNDU);
    mpfr_add (r->rad, r->rad, *aux, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
