
/******************************************************************************
* MODULE     : interval_exp.cpp
* DESCRIPTION: Compute the exponential of an interval
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
exp (const interval& x) {
  if (is_error (x)) return x;
  if (has_low_prec (x)) {
    interval_rep* r= raw_interval ();
    mpfr_exp (r->min, x->min, GMP_RNDD);
    mpfr_exp (r->max, x->max, GMP_RNDU);
    return r;
  }
  else if (radius (x) >= error (1, -DEFAULT_LOW - 1)) {
    interval_rep* r= raw_interval ();
    direct_dyadic aux (DEFAULT_LOW << 1);
    mpfr_sub (*aux, x->mid, x->rad, GMP_RNDD);
    mpfr_exp (r->min, *aux, GMP_RNDD);
    mpfr_add (*aux, x->mid, x->rad, GMP_RNDU);
    mpfr_exp (r->max, *aux, GMP_RNDU);
    return r;
  }
  else {
    mp_prec_t exp_prec= mpfr_sgn (x->rad) == 0?
      mpfr_get_prec (x->mid): 1 - mpfr_get_step (x->rad);
    interval_rep* r= raw_interval (exp_prec);
    direct_dyadic err;
    mpfr_exp (r->mid, x->mid, GMP_RNDN);
    CHECK_SIGNED_UNDERFLOW (r, 1);
    CHECK_OVERFLOW (r);
    mpfr_mul (*err, r->mid, x->rad, GMP_RNDU);
    mpfr_set_ui_2si (r->rad, 2, mpfr_get_step (*err), GMP_RNDU);
    mpfr_add (r->rad, *err, r->rad, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
