
/******************************************************************************
* MODULE     : interval_log.cpp
* DESCRIPTION: Compute the log of an interval
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
log (const interval& x) {
  if (is_error (x)) return x;
  if (sign (lower (x)) < 0) return interval::nanin;
  if (is_infinity (x)) return x;
  if (has_low_prec (x)) {
    interval_rep* r= raw_interval ();
    mpfr_log (r->min, x->min, GMP_RNDD);
    mpfr_log (r->max, x->max, GMP_RNDU);
    return r;
  }
  else {
    double lambda= abs ((double) expo (x)) + 1.0;
    mp_exp_t gain= (mp_exp_t) (log (lambda) / log (2.0));
    interval_rep* r= raw_interval (prec (x) + gain);
    direct_dyadic err;
    mpfr_log (r->mid, x->mid, GMP_RNDN);
    mpfr_set_ui_2si (*err, 3, mpfr_get_step (x->mid), GMP_RNDU);
    mpfr_add (*err, x->rad, *err, GMP_RNDU);
    mpfr_div (r->rad, *err, x->mid, GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
