
/******************************************************************************
* MODULE     : interval_out.cpp
* DESCRIPTION: Outputting intervals
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hpp"
#include "interval_impl.hpp"
namespace mml {

ostream&
operator << (ostream& out, const interval& x) {
  if (is_error (x)) return out << "NanIn";
  if (is_infinity (x) && x>0) return out << "Infty";
  if (is_infinity (x) && x<0) return out << "-Infty";
  if (interval_by_bounds)
    return out << "[ " << lower (x) << ", " << upper (x) << " ]";
  if (is_fuzzy (x)) return out << "Fuzz";
  dyadic mid= center (x);
  dyadic rad= radius (x);
  if (is_zero (rad)) return out << mid;
  if (is_zero (mid)) return print_zero (out, *rad);
  mp_exp_t bits= max ((mp_exp_t) expo (mid) - expo (rad) - 1,
		      (mp_exp_t) MIN_PREC);
  return print (out, *mid, min (prec (mid), bits));
}

generic
as_generic (const interval& x) {
  if (is_error (x)) return "NanIn";
  if (is_infinity (x))
    return x>0? generic ("Infty"): negate (generic ("Infty"));
  if (interval_by_bounds)
    return as_generic ("[]", lower (x), upper (x));
  return print_to_string (x);
}

} // namespace mml
