
/******************************************************************************
* MODULE     : interval_segment.cpp
* DESCRIPTION: Construct an interval from its lower and upper limits
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
segment (const dyadic& min, const dyadic& max) {
  if (has_low_prec (min) || has_low_prec (max) ||
      (abs (min) <= ((max - min) << DEFAULT_LOW))) {
    interval_rep* r= raw_interval ();
    mpfr_set (r->min, *min, GMP_RNDD);
    mpfr_set (r->max, *max, GMP_RNDU);
    return r;
  }
  else {
    interval_rep* r= raw_interval (prec (min));
    mpfr_mid (r->mid, *min, *max, GMP_RNDN);
    mpfr_rad (r->rad, *min, *max, GMP_RNDU);
    return r;
  }
}

} // namespace mml
