
/******************************************************************************
* MODULE     : interval_truncate.cpp
* DESCRIPTION: Truncate the interval to a given precision
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
truncate (const interval& x, mp_prec_t new_prec) {
  if (prec (x) <= new_prec || is_epsilon (x) ||
      is_error (x) || is_fuzzy (x)) return x;
  if (prec (x) <= DEFAULT_LOW) {
    direct_dyadic cen, rho, new_rho;
    mpfr_sub (*rho, x->max, x->min, GMP_RNDU);
    mp_exp_t e;
    if (mpfr_sgn (*rho) == 0) e= mpfr_get_exp (x->min);
    else {
      e= mpfr_get_exp (*rho);
      if (mpfr_sgn (x->min) != 0) e= max (e, mpfr_get_exp (x->min));
      if (mpfr_sgn (x->max) != 0) e= max (e, mpfr_get_exp (x->max));
    }
    mpfr_mid (*cen, x->min, x->max, GMP_RNDN);
    mpfr_set_ui_2si (*new_rho, 1, e - new_prec, GMP_RNDU);
    if (mpfr_cmp (*new_rho, *rho) < 0) return x;
    interval_rep* r= raw_interval ();
    mpfr_sub (r->min, *cen, *new_rho, GMP_RNDD);
    mpfr_add (r->max, *cen, *new_rho, GMP_RNDU);
    return r;
  }
  else {
    interval_rep* r= raw_interval (max (new_prec, DEFAULT_LOW));
    mpfr_set_ui_2si (r->rad, 1, mpfr_get_exp (x->mid) - new_prec, GMP_RNDU);
    if (new_prec <= DEFAULT_LOW) {
      mpfr_sub (r->min, x->mid, r->rad, GMP_RNDD);
      mpfr_add (r->max, x->mid, r->rad, GMP_RNDU);
      return r;
    }
    else {
      mpfr_set (r->mid, x->mid, GMP_RNDN);
      return r->finish_high ();
    }
  }
}

} // namespace mml
