
/******************************************************************************
* MODULE     : interval_wider.cpp
* DESCRIPTION: Is the radius of an interval is larger than a given number?
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

bool
wider (const interval& x, const dyadic& rad) {
  // A non zero exact result may lead to subtle errors, because its
  // bit precision is unknown (typically lost after normalization).
  // In the current test, we therefore replace such intervals by
  // slightly larger inexact surrounding intervals.
  if (has_low_prec (x)) {
    direct_dyadic aux;
    mpfr_rad (*aux, x->min, x->max, GMP_RNDU);
    if (mpfr_sgn (*aux) == 0 && mpfr_sgn (x->min) != 0) {
      mpfr_set (*aux, x->max, GMP_RNDU);
      mpfr_nextabove (*aux);
      mpfr_sub (*aux, *aux, x->min, GMP_RNDU);
    }
    return mpfr_cmp (*aux, *rad) >= 0;
  }
  else {
    if (mpfr_sgn (x->rad) == 0) {
      direct_dyadic aux;
      mpfr_set_ui_2si (*aux, 1, step (x->mid), GMP_RNDN);
      return mpfr_cmp (*aux, *rad) >= 0;
    }
    else return mpfr_cmp (x->rad, *rad) >= 0;
  }
}

} // namespace mml
