
/******************************************************************************
* MODULE     : approx_roots.hpp
* DESCRIPTION: Finding the smallest root of a complex polynomial
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_APPROX_ROOTS_HPP
#define __MML_APPROX_ROOTS_HPP
#include "complexify.hpp"
#include "dense_polynomial.hpp"
namespace mml {
#define TMPL template<typename R, typename V>
#define Complex complexify<R>
#define Polynomial polynomial<Complex,dense<V> >

TMPL R
L1_norm (const Polynomial& P) {
  nat i, n= dim (P);
  R tot= 0;
  for (i=0; i<n; i++)
    tot += abs (P[i]);
  return tot;
}

TMPL R
max_norm (const Polynomial& P) {
  nat i, n= dim (P);
  R res= 0;
  for (i=0; i<n; i++)
    res= max (res, abs (P[i]));
  return res;
}

inline nat
number_iterates (nat d) {
  if (d <= 1) return 20;
  else return number_iterates (d >> 1) + 2;
}

TMPL R
lower_bound_roots (const Polynomial& P) {
  if (P[0] == 0) return 0;
  // if (deg (P) == 0) return R::plus_infinity;
  nat i, d= deg (P), l= number_iterates (d);
  Polynomial Q= reverse (P);
  Q /= Q [deg (Q)];
  cout << "Q\t" << Q << "\n";
  Polynomial A (1, 1);
  R inv_r= 1, n= 1;
  for (i=0; i<l; i++, n *= 2) {
    if (i>0) {
      R scale= max_norm (A);
      A /= Complex (scale);
      inv_r *= exp (log (scale) / n);
    }
    cout << "Test\t" << Q/A << "\n";
    Polynomial B= (A * Polynomial (1, 1)) % Q;
    cout << "Test\t" << Q/B << "\n";
    A= (A * A) % Q;
    R tot= L1_norm (A) * abs (Q[0]);
    R aux= inv_r * exp (log (tot) / n);
    cout << i+1 << "\t" << A << "\n\t" << inv_r << ", " << 1/aux << "\n";
  }
  R tot= abs (Q[0]) * L1_norm (A);
  inv_r *= exp (log (tot) / n);
  return 1/inv_r;
}

#undef TMPL
#undef Complex
#undef Polynomial
} // namespace mml
#endif // __MML_APPROX_ROOTS_HPP
