
/******************************************************************************
* MODULE     : complex.hpp
* DESCRIPTION: Complex numbers
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_COMPLEX_HPP
#define __MML_COMPLEX_HPP
#include "real.hpp"
#include "complexify.hpp"
namespace mml {
typedef complexify<real> complex;
typedef complexify<interval> complex_interval;

inline dyadic weight (const complex& z) {
  return weight (Re (z), Im (z)); }
inline bool sq_wider (const complex_interval& z, const dyadic& err) {
  return wider (Re (z), err) || wider (Im (z), err); }
inline dyadic sq_radius (const complex_interval& z) {
  return max (radius (Re (z)), radius (Im (z))); }
inline complex_interval sq_approx (const complex& z, const dyadic& err) {
  return complex_interval (approx (Re (z), err), approx (Im (z), err)); }
inline dyadic sq_bound (const complex& z) {
  return max (bound (Re (z)), bound (Im (z))); }

/******************************************************************************
* Extra var_complex class on the same model as the real numbers
******************************************************************************/

class var_complex_rep REP_STRUCT {
public:
  const dyadic wt;        // weight of real number as an expression
  mp_exp_t magn;          // order of magnitude of number
  complex_interval best;  // the best currently available approximation

public:
  inline var_complex_rep (const dyadic& wt2):
    wt (wt2), best (complex_interval (interval::fuzzy, interval::fuzzy)) {}
  inline var_complex_rep (const dyadic& wt2, const complex_interval& best2):
    wt (wt2), best (best2) {}
  virtual inline ~var_complex_rep () {}
  virtual generic expression () const = 0;
  virtual void sq_approx (const dyadic& err) = 0;
  inline void init_magn () { magn= max (expo (Re (best)), expo (Im (best))); }
  inline void init () {
    sq_approx (lp_plus_infinity);
    magn= max (expo (Re (best)), expo (Im (best))); }
};

class var_complex {
INDIRECT_PROTO (var_complex, var_complex_rep)
public:
  friend dyadic sq_bound (const var_complex& x);
  friend dyadic bound (const var_complex& x);
  friend complex_interval sq_approx (const var_complex& x, const dyadic& err);
  friend complex_interval approx (const var_complex& x, const dyadic& err);
  inline operator complex () const;
};
INDIRECT_IMPL (var_complex, var_complex_rep)

real Re (const var_complex& z);
real Im (const var_complex& z);
inline dyadic weight (const var_complex& z) {
  return z->wt; }
inline var_complex::operator complex () const {
  return complex (Re (*this), Im (*this)); }
inline generic as_generic (const var_complex& z) {
  if (mml_default_expr) return z->expression ();
  else return as_generic ((complex) z); }
inline ostream& operator << (ostream& out, const var_complex& z) {
  return out << as_generic (z); }

} // namespace mml
#endif // __MML_COMPLEX_HPP
