
/******************************************************************************
* MODULE     : complex.cpp
* DESCRIPTION: Effective complex numbers based on same model as reals
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "complex.hpp"
namespace mml {

/******************************************************************************
* Different types of approximations
******************************************************************************/

dyadic
sq_bound (const var_complex& x) {
  if ((is_signed (Re (x.rep->best)) || is_epsilon (Re (x.rep->best))) &&
      (is_signed (Im (x.rep->best)) || is_epsilon (Im (x.rep->best))))
    return max (bound (Re (x.rep->best)), bound (Im (x.rep->best)));
  else return dyadic (1, x.rep->magn, DEFAULT_LOW);
}

dyadic
bound (const var_complex& x) {
  if ((is_signed (Re (x.rep->best)) || is_epsilon (Re (x.rep->best))) &&
      (is_signed (Im (x.rep->best)) || is_epsilon (Im (x.rep->best))))
    return hypot (bound (Re (x.rep->best)), bound (Im (x.rep->best)));
  else return dyadic (1.414213563) * dyadic (1, x.rep->magn, DEFAULT_LOW);
}

complex_interval
sq_approx (const var_complex& x, const dyadic& err) {
  if (sq_wider (x.rep->best, err)) {
    x.rep->sq_approx (err);
    // The precision of a computed approximation can be a little bit too low
    // due to rounding errors. In that case, we increase the precision.
    // If the computed precision is far too low, then this probably indicates
    // a bug in the code.
    if (sq_wider (x.rep->best, err)) {
      dyadic factor= sq_radius (x.rep->best) / err;
      if (factor > 16) {
	cerr << "\nPrecision deficiency: " << factor << "\n";
	assert (factor <= 16);
      }
      return sq_approx (x, err / (factor + 1));
    }
  }
  return x.rep->best;
}

complex_interval
approx (const var_complex& x, const dyadic& err) {
  return sq_approx (x, err / dyadic (1.414213563));
}

}
