
/******************************************************************************
* MODULE     : real_acos.cpp
* DESCRIPTION: Compute the arc tangent of a real number
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class acos_real_rep: public real_rep {
private:
  real   x;
  dyadic inv_der;
public:
  inline acos_real_rep (const real& x2, const dyadic& i2):
    real_rep (weight (x2)), x (x2), inv_der (i2) { init (); }
  generic expression () const { return as_generic ("arccos", x); }
  void approx (const dyadic& err) {
    best= acos (approx_mult (x, mul_lo (err, inv_der))); }
};

real
acos (const real& x) {
  real y= real (1) - square (x);
  assert (y >= 0); // -1 <= x <= 1
  if (y == 0) return x > 0? real (0): real::pi ();
  return real (new acos_real_rep (x, lower (expand (sqrt (y)))));
}

}
