
/******************************************************************************
* MODULE     : real_atan.cpp
* DESCRIPTION: Compute the arc tangent of a real number
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class atan_real_rep: public real_rep {
private:
  real x;
public:
  inline atan_real_rep (const real& x2):
    real_rep (weight (x2)), x (x2) { init (); }
  generic expression () const { return as_generic ("arctan", x); }
  void approx (const dyadic& err);
};

void
atan_real_rep::approx (const dyadic& err) {
  dyadic bnd= lower (expand (abs (x), 2));
  if (bnd > (dyadic (1) << 100)) bnd= dyadic (1) << 100;
  dyadic mul= add_lo (mul_lo (bnd, bnd), sint_lo (1, DEFAULT_LOW));
  dyadic eps= mul_lo (err, mul);
  best= atan (approx_mult (x, eps));
}

real
atan (const real& x) {
  return real (new atan_real_rep (x));
}

}
