
/******************************************************************************
* MODULE     : real_atan.cpp
* DESCRIPTION: Variant of atan for computing the argument of a complex number
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

real
atan2 (const real& y, const real& x) {
  assert (max (abs (x), abs (y)) > 0);
  if (sign (lower (x->best)) > 0) return atan (y/x);
  if (sign (lower (y->best)) > 0) return atan (-x/y) + (real::pi () >> 1);
  if (sign (upper (y->best)) < 0) return atan (-x/y) - (real::pi () >> 1);
  if (sign (upper (x->best)) < 0) return atan (y/x) + real::pi ();
  assert (false); // this situation should never occur
  return 0;
}

}
