
/******************************************************************************
* MODULE     : real_constants.cpp
* DESCRIPTION: Various mathematical constants
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

typedef interval (*approx_algo) (mp_prec_t);

class constant_real_rep: public real_rep {
private:
  approx_algo algo;
  generic name;
public:
  inline constant_real_rep (approx_algo algo2, const generic& name2):
    real_rep (dyadic (1), algo2 (DEFAULT_LOW)),
    algo (algo2), name (name2) { init_magn (); }
  generic expression () const { return name; }
  void approx (const dyadic& err);
};

void
constant_real_rep::approx (const dyadic& err) {
  mp_prec_t required= (mp_prec_t) (2 + expo (best) - expo (err));
  mp_prec_t propose = prec (best);
  if (propose < (BITS_PER_LIMB << 8)) propose += (propose >> 1);
  else propose <<= 1;
  best= algo (max (required, propose));
}

real real::log2 () {
  return new constant_real_rep (interval::log2,
				as_generic ("log", generic ("2"))); }
real real::pi () {
  return new constant_real_rep (interval::pi, "mathpi"); }
real real::euler () {
  return new constant_real_rep (interval::euler, "matheuler"); }
//real real::catalan () {
//  return new constant_real_rep (interval::euler, "mathcatalan"); }

}
