
/******************************************************************************
* MODULE     : real_cos.cpp
* DESCRIPTION: Compute the cosine of a real number
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class cos_real_rep: public real_rep {
private:
  real x;
public:
  inline cos_real_rep (const real& x2):
    real_rep (weight (x2)), x (x2) { init (); }
  generic expression () const { return as_generic ("cos", x); }
  void approx (const dyadic& err);
};

void
cos_real_rep::approx (const dyadic& err) {
  if (err >= dyadic (6.2832))
    best= segment (sint_lo (-1, DEFAULT_LOW), sint_up (1, DEFAULT_LOW));
  else {
    interval xa;
    if (err >= error (1)) best= cos (approx_mult (x, err));
    else {
      dyadic eps= error (1);
      dyadic aux= bound (sin (mml::approx (x, eps)));
      best= cos (approx_mult (x, min (div_lo (err, aux), eps)));
    }
  }
}

real
cos (const real& x) {
  return real (new cos_real_rep (x));
}

}
