
/******************************************************************************
* MODULE     : real_dyadic.cpp
* DESCRIPTION: Dyadic numbers as effective real numbers
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "string.hpp"
#include "real.hpp"
namespace mml {

class dyadic_real_rep: public real_rep {
  dyadic x;
public:
  inline dyadic_real_rep (const dyadic& x2):
    real_rep (dyadic (1, DEFAULT_LOW), as_interval (x2)),
    x (x2) { init_magn (); }
  generic expression () const { return as_generic (x); }
  void approx (const dyadic& err);
};

void
dyadic_real_rep::approx (const dyadic& err) {
  if (x == 0) return;
  mp_prec_t required= (mp_prec_t) (2 + expo (best) - expo (err));
  mp_prec_t new_prec= max (required, prec (best) << 1);
  best= extend (best, new_prec);
}

real as_real (double x) { return new dyadic_real_rep (dyadic (x)); }
real as_real (const dyadic& x) { return new dyadic_real_rep (x); }

}
