
/******************************************************************************
* MODULE     : real_exp.cpp
* DESCRIPTION: Compute the exponential of a real number
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class exp_real_rep: public real_rep {
private:
  real x;
public:
  inline exp_real_rep (const real& x2):
    real_rep (weight (x2), exp (approx_mult (x2, error (1, -2)))),
    x (x2) { init (); }
  generic expression () const { return as_generic ("exp", x); }
  void approx (const dyadic& err) {
    dyadic eps= div_lo (err, upper (best));
    best= exp (approx_mult (x, eps));
  }
};

real
exp (const real& x) {
  static real max_exp_argument= as_real (log (dyadic::largest ()));
  assert (abs (x) < max_exp_argument);
  return real (new exp_real_rep (x));
}

}
