
/******************************************************************************
* MODULE     : real_integer.cpp
* DESCRIPTION: Integers as effective real numbers
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class integer_real_rep: public real_rep {
  integer i;
public:
  inline integer_real_rep (const integer& i2):
    real_rep (dyadic (1, DEFAULT_LOW), as_interval (i2)),
    i (i2) { init_magn (); }
  generic expression () const { return as_generic (i); }
  void approx (const dyadic& err);
};

void
integer_real_rep::approx (const dyadic& err) {
  if (i == 0) return;
  mp_prec_t required= (mp_prec_t) (2 + expo (best) - expo (err));
  mp_prec_t new_prec= max (required, prec (best) << 1);
  best= extend (best, new_prec);
}

real::real (const integer& i): rep (new integer_real_rep (i)) {}

}
