
/******************************************************************************
* MODULE     : real_round.cpp
* DESCRIPTION: Rounding real numbers to integers
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

static integer
round (const real& x, int max_cancel, int direction) {
  integer i;
  dyadic r= center (approximate (x, 1e-20));
  switch (direction) {
  case -1: i= as_integer (floor (r)); break;
  case  0: i= as_integer (trunc (r)); break;
  case  1: i= as_integer (ceil  (r)); break;
  }
  int sgn= sign (x-i, max_cancel);
  switch (direction) {
  case -1: return sgn<0? i - integer(1): i;
  case  0:
    if (sgn<0 && i>0) return i - integer (1);
    else if (sgn>0 && i<0) return i + integer (1);
    else return i;
  case  1: return sgn>0? i + integer (1): i;
  }
  return i;
}

integer floor (const real& x, int max_cancel) {
  return round (x, max_cancel, -1); }
integer trunc (const real& x, int max_cancel) {
  return round (x, max_cancel, 0); }
integer ceil  (const real& x, int max_cancel) {
  return round (x, max_cancel, 1); }
integer round (const real& x, int max_cancel) {
  return round (x + real (rational (1) / rational (2)), max_cancel, -1); }

}
