
/******************************************************************************
* MODULE     : real_sqrt.cpp
* DESCRIPTION: Compute the square root of a real number
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class sqrt_real_rep: public real_rep {
private:
  real x;
public:
  inline sqrt_real_rep (const real& x2):
    real_rep (weight (x2), sqrt (expand (x2, DEFAULT_LOW))),
    x (x2) { init (); }
  generic expression () const { return as_generic ("sqrt", x); }
  void approx (const dyadic& err) {
    dyadic eps= mul_lo (err, lower (best)) << 1;
    best= sqrt (approx_mult (x, eps)); }
};

real
sqrt (const real& x) {
  assert (x >= 0);
  if (normalize (x, MAX_CANCEL_TEST, error (1))) return 0;
  return real (new sqrt_real_rep (x));
}

}
