
/******************************************************************************
* MODULE     : generic_atom.cpp
* DESCRIPTION: Atomic generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "generic_internal.hpp"
namespace mml {

class generic_atom_rep: public generic_rep {
public:
  symbol<string> data;

protected:
  nat get_type () const { return GENERIC_ATOM; }
  nat get_dim () const { return 0; }
  generic get_child (nat i) const {
    assert (false); // illegal access
    return generic (); }
  nat get_hash_value () const { return hash (data); }
  bool identical (const generic& g) const {
    if (type (g) != GENERIC_ATOM) return false;
    return data == ((generic_atom_rep*) inspect (g)) -> data; }
  generic flatten (bool recursive) const {
    (void) recursive;
    return generic_atom (*data); }

public:
  generic_atom_rep (const string& data2): data (data2) {}
};

generic::generic ():
  rep (new generic_atom_rep (string (""))) {}

generic::generic (int i):
  rep (new generic_atom_rep (as_string (i))) {}

generic::generic (char* s):
  rep (new generic_atom_rep (string (s))) {}

generic::generic (const string& s):
  rep (new generic_atom_rep (s)) {}

generic
generic_atom (char* s) {
  return new generic_atom_rep (string (s));
}

generic
generic_atom (const string& s) {
  return new generic_atom_rep (s);
}

symbol<string>
as_atom (const generic& g) {
  return ((generic_atom_rep*) inspect (g)) -> data;
}

string
as_string (const generic& g) {
  return * ((generic_atom_rep*) inspect (g)) -> data;
}

} // namespace mml
